package at.technikumwien.movies;

import javax.annotation.Resource;
import javax.ejb.*;
import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.PersistenceContext;
import java.util.List;
import java.util.logging.Logger;

@Stateless
@TransactionManagement(value=TransactionManagementType.CONTAINER)
public class MoviesService {
    private static final Logger LOGGER = Logger.getLogger(MoviesService.class.getName());

    @Resource
    private SessionContext context;

    @PersistenceContext(unitName = "MoviesPU")
    private EntityManager em;

    public Movie findById(long id) {
        LOGGER.info("findById() >> id=" + id);

        Movie movie = em.find(Movie.class, id);
        if (movie == null) {
            throw new EntityNotFoundException("can't find movie with id=" + id);
        }

        return movie;
    }

    public List<Movie> findByTitle(String title) {
        LOGGER.info("findByTitle() >> title=" + title);

        return em.createNamedQuery("Movies.selectByTitle", Movie.class)
                .setParameter("title", "%" + title + "%")
                .getResultList();
    }

    public List<Movie> findAll() {
        LOGGER.info("findAll()");

        return em.createNamedQuery("Movies.selectAll", Movie.class) //JPQL -> java persistence query language
                .getResultList();
    }

    public void removeById(long id) {
        LOGGER.info("removeById() >> id=" + id);

        Movie movie = findById(id);
        em.remove(movie); //managed news required
    }

    public List<Actor> findAllActors() {
        LOGGER.info("findAllActors)");

        return em.createNamedQuery("Actors.selectAllActors", Actor.class)
                .getResultList();
    }

    public List<Studio> findAllStudios() {
        LOGGER.info("findAllStudios)");

        return em.createNamedQuery("Studios.selectAllStudios", Studio.class)
                .getResultList();
    }

    // TODO maybe check if the movie already exists?
    @TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
    public void save(List<Movie> movies) {

        try {
            for (Movie movie : movies) {
                // Get studio for the movie from the database
                Studio movieStudio = movie.getStudio();
                LOGGER.info("Get a studio" + movieStudio);
                List<Studio> studioListResult = em.createNamedQuery("Studios.getIdByProperties", Studio.class)
                        .setParameter("name", movieStudio.getName())
                        .setParameter("countrycode", movieStudio.getCountrycode())
                        .setParameter("postcode", movieStudio.getPostcode())
                        .getResultList();

                if (!studioListResult.isEmpty()) {
                    // Take the first element in the list
                    Long studioId = studioListResult.get(0).getId();
                    movieStudio.setId(studioId);
                }
                else
                {
                    LOGGER.info("Rollback! Movie studio: " + movieStudio);
                    context.setRollbackOnly();
                    return;
                }

                // Get actor for the movie from the database
                for (Actor movieActor : movie.getActors()) {
                    LOGGER.info("Get an actor " + movieActor);
                    List<Actor> actorListResult = em.createNamedQuery("Actors.getIdByProperties", Actor.class)
                            .setParameter("firstname", movieActor.getFirstname())
                            .setParameter("lastname", movieActor.getLastname())
                            .setParameter("sex", movieActor.getSex())
                            .setParameter("birthdate", movieActor.getBirthdate())
                            .getResultList();

                    if (!actorListResult.isEmpty()) {
                        // Take the first element in the list
                        Long actorId = actorListResult.get(0).getId();
                        movieActor.setId(actorId);
                    }
                    else
                    {
                        LOGGER.info("Rollback! Movie actor: " + movieActor);
                        context.setRollbackOnly();
                        return;
                    }
                }
            }

            LOGGER.info("save() >> movies" + movies);
            for (Movie movieToImport : movies) {
                em.merge(movieToImport);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}